{% extends 'base.html' %}
{% load static %}

{% block content %}
<ul class="list-group">
	<li class="list-group-item list-item centered list-header">
		Streaming seems to be disabled :(
	</li>
</ul>
{% endblock %}
