# Generated by Django 2.2 on 2021-07-11 17:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("core", "0012_currentsong_last_paused")]

    operations = [
        migrations.AlterField(
            model_name="queuedsong",
            name="external_url",
            field=models.CharField(max_length=2000),
        ),
        migrations.AlterField(
            model_name="queuedsong",
            name="internal_url",
            field=models.CharField(blank=True, max_length=2000, null=True),
        ),
        migrations.AlterField(
            model_name="queuedsong",
            name="stream_url",
            field=models.CharField(blank=True, max_length=2000, null=True),
        ),
        migrations.AlterField(
            model_name="currentsong",
            name="external_url",
            field=models.CharField(max_length=2000),
        ),
        migrations.AlterField(
            model_name="currentsong",
            name="stream_url",
            field=models.CharField(blank=True, max_length=2000, null=True),
        ),
    ]
