# Generated by Django 2.2.20 on 2021-04-27 11:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("core", "0010_tag_active")]

    operations = [
        migrations.AddField(
            model_name="currentsong",
            name="stream_url",
            field=models.CharField(blank=True, max_length=2000),
        ),
        migrations.AddField(
            model_name="queuedsong",
            name="stream_url",
            field=models.CharField(blank=True, max_length=2000),
        ),
    ]
