# Generated by Django 2.2.10 on 2020-05-14 10:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("core", "0007_remove_playlistentry_song")]

    operations = [
        migrations.AlterField(
            model_name="archivedplaylist",
            name="list_id",
            field=models.CharField(max_length=2000),
        ),
        migrations.AlterField(
            model_name="archivedsong",
            name="url",
            field=models.CharField(max_length=2000, unique=True),
        ),
        migrations.AlterField(
            model_name="currentsong",
            name="external_url",
            field=models.CharField(blank=True, max_length=2000),
        ),
        migrations.AlterField(
            model_name="currentsong",
            name="internal_url",
            field=models.CharField(max_length=2000),
        ),
        migrations.AlterField(
            model_name="playlistentry",
            name="url",
            field=models.CharField(max_length=2000),
        ),
        migrations.AlterField(
            model_name="queuedsong",
            name="external_url",
            field=models.CharField(blank=True, max_length=2000),
        ),
        migrations.AlterField(
            model_name="queuedsong",
            name="internal_url",
            field=models.CharField(max_length=2000),
        ),
    ]
