# Generated by Django 2.2.19 on 2021-04-26 22:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("core", "0009_delete_pad")]

    operations = [
        migrations.AddField(
            model_name="tag",
            name="active",
            field=models.BooleanField(default=True),
            preserve_default=False,
        )
    ]
